/**************************************************
 * Off-printer Bed Levelling Sensor test 
 * for BLtouch/3Dtouch and clones
 * 
 * Connect:
 * ZMIN to A0
 * GND to GND
 * SIG to D9
 * VDD to 5V
 * GND to GND
 * 
 * Set baud rate to 19200 and issue single letter 
 * commands as required to test all modes.
 * See https://www.antclabs.com/bltouch for details.
 **************************************************/

#include <Servo.h>
Servo blt;

const char cmdn[][10]={
  "Deploy", 
  "Tswitch", 
  "Retract",
  "Self test", 
  "Alarm off" 
};
const int cmdp[]={650, 1165, 1475, 1780, 2190};
int idx,inByte,cc;
bool curr,prev;
unsigned long cts,pts;

void setup() {
    // start serial port at 19200 bps and wait for port to open:
  Serial.begin(19200);
  while (!Serial); // wait for serial port to connect. Needed for native USB port only
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(A0, INPUT_PULLUP);

  Serial.println("*** BLtouch/3Dtouch probe tester ***");

  char t[]="? = ";
  for(idx=0; idx<=4; idx++){
    t[0]=cmdn[idx][0];
    Serial.print(t),Serial.println(cmdn[idx]);
  }
  Serial.println("? = read ZMIN");
  Serial.println("The Arduino LED echoes the ZMIN state, ON=High / OFF=Low");
}
void loop() {
  if (Serial.available() > 0) {
    // get incoming byte:
    inByte = Serial.read();
    switch(inByte){
      case 'D':
      case 'd':
        idx=0;
        break;
      case 'T':
      case 't':
        idx=1;
        break;      
      case 'R':
      case 'r':
        idx=2;
        break;
      case 'S':
      case 's':
        idx=3;
        break;
      case 'A':
      case 'a':
        idx=4;
        break;
        case '?':
        Serial.println();
        Serial.print("ZMIN is "),Serial.print(digitalRead(A0)?"High":"Low");
        Serial.print(" ("),Serial.print(analogRead(A0)*5.0/1024.0),Serial.println("V)");
        idx=-1;
        cc=0;
        break;
      default:
        idx=-1;         
    }
    if(idx>=0){
      Serial.println();
      Serial.println(cmdn[idx]);
      blt.attach(9);
      blt.writeMicroseconds(cmdp[idx]);
      delay(200);
      blt.detach();
      idx=-1;
      cc=0;
    }
  }
  curr=digitalRead(A0);
  cts=millis();
  if((curr!=prev)||(cts-pts>100)){
    digitalWrite(LED_BUILTIN,curr);
    Serial.print(curr?"#":"_");
    if(cc++>=80){
      Serial.println();
      cc=0;
    }
    prev=curr;
    pts=cts;
  }
}
